var path = require('path');
var fs = require('fs');
var utils = require('./utils');
var webpack = require('webpack');
var config = require('../config');
var vueLoaderConfig = require('./vue-loader.conf');
var CopyWebpackPlugin = require('copy-webpack-plugin');
var source = config.common.assetsSourcePath;
var publicSource= config.common.assetsPublicSourcePath;
var componentsSource = config.common.assetsComponentsSourcePath;
var publicDirectory = config.common.publicDirectory;
var env = process.env.NODE_ENV;
var args = JSON.parse(process.env.npm_config_argv).original || process.argv;
var mode = args[1];
var index = args.findIndex(a => a === '-m') + 1;
var modules = index ? args[index] : [];

var files = fs.readdirSync('./src');
var entry = {};
var copyPlugins = [];

function copyAssetsPlugin(mod) {
  copyPlugins.push(new CopyWebpackPlugin([{
    context: `./src/${mod}/`,
    from: 'asset*/**',
    to: `${config[env].assetsRoot}/${mod}/`
  }, {
    context: `./src/${mod}/`,
    from: '*.xml',
    to: `${config[env].assetsRoot}/${mod}/`
  }], {
    ignore: [
      '*.js', '*.vue', 'font_example_*.png'
    ]
  }));
}

function add(d) {
  if (d.indexOf('.') > -1) return;
  entry[d] = `./src/${d}/index.js`;
  copyAssetsPlugin(d);
}

if (mode !== 'dll' && modules.length) {
  modules = modules.split(',');
  if (mode.indexOf('dev') !== -1) modules.push(publicDirectory);
  modules.map(d => {
    if (files.indexOf(d) === -1) throw new Error(`module not found! [ ${d} ]`);
    add(d);
  });
} else {
  files.map(d => add(d));
}

if (entry[publicDirectory]) {
  entry[publicDirectory] = ['public/js/base/main'];
  copyPlugins.push(new CopyWebpackPlugin([{
    context: publicSource,
    from: 'dll',
    to: `${config[env].assetsRoot}/${publicDirectory}/`,
    ignore: ['*.json']
  }]));
}

module.exports = {
  entry: entry,
  output: {
    path: config[env].assetsRoot,
    filename: '[name]/[name].js',
    publicPath: config[env].assetsPublicPath
  },
  resolve: {
    extensions: ['.js', '.vue', '.json'],
    alias: {
      'vue$': 'vue/dist/vue.esm.js',
      'vue-router': 'public/js/lib/vue-router',
      'src': source,
      'public': publicSource,
      'components': componentsSource
    },
    modules: [
      'node_modules'
    ]
  },
  module: {
    rules: [
      {
        test: /\.(js|vue)$/,
        loader: 'eslint-loader',
        enforce: 'pre',
        include: [source],
        exclude: [/node_modules/],
        options: {
          formatter: require('eslint-friendly-formatter')
        }
      },
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: vueLoaderConfig
      },
      {
        test: /\.js$/,
        loader: 'babel-loader',
        include: [source],
        exclude: [/node_modules/]
      },
      {
        test: /\.html$/,
        loader: 'vue-html-loader'
      },
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          name: utils.assetsPath('assets/images/[name].[ext]')
        }
      },
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        query: {
          limit: 10000,
          name: utils.assetsPath('assets/fonts/[name].[ext]')
        }
      }
    ]
  },
  // Assets
  plugins: copyPlugins
};
