var utils = require('./utils');
var webpack = require('webpack');
var config = require('../config');
var merge = require('webpack-merge');
var baseWebpackConfig = require('./webpack.base.conf');
var HtmlWebpackPlugin = require('html-webpack-plugin');
var ExtractTextPlugin = require('extract-text-webpack-plugin');
var FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin');
// add hot-reload related code to entry chunks
Object.keys(baseWebpackConfig.entry).forEach(function(name) {
  baseWebpackConfig.entry[name] = ['./bin/dev-client'].concat(baseWebpackConfig.entry[name]);
});

var modules = Object.keys(baseWebpackConfig.entry);
var webpackConfig = merge(baseWebpackConfig, {
  module: {
    rules: utils.styleLoaders({
      sourceMap: config.dev.cssSourceMap,
      extract: true
    })
  },
  // cheap-module-eval-source-map is faster for development
  devtool: '#cheap-module-eval-source-map',
  plugins: [
    new webpack.DefinePlugin({
      'process.env': config.dev.env
    }),
    // extract css into its own file
    new ExtractTextPlugin({
      filename: utils.assetsPath('../[name]/[name].css')
    }),
    // https://github.com/glenjamin/webpack-hot-middleware#installation--usage
    new webpack.HotModuleReplacementPlugin(),
    new webpack.NoEmitOnErrorsPlugin(),
    // https://github.com/ampedandwired/html-webpack-plugin
    new FriendlyErrorsPlugin(),
    new HtmlWebpackPlugin({
      filename: 'index.html',
      template: 'dev.tpl',
      inject: false,
      modulename: modules.filter(a => a !== config.common.publicDirectory)
    }),
    // Dll library
    ...Object.keys(config.build.library).map(name => {
      return new webpack.DllReferencePlugin({
        context: '.',
        manifest: require(`${config.common.assetsPublicSourcePath}/dll/${name}.dll.manifest.json`)
      })
    })
  ]
});

for (var key in webpackConfig.entry) {
  webpackConfig.plugins.push(
    new HtmlWebpackPlugin({
      filename: key + '/index.html',
      template: 'index.html',
      chunks: [key],
      inject: true
    })
  );
}
module.exports = webpackConfig;
