var path = require('path');
var utils = require('./utils');
var webpack = require('webpack');
var config = require('../config');
var merge = require('webpack-merge');
var baseWebpackConfig = require('./webpack.base.conf');
var HtmlWebpackPlugin = require('html-webpack-plugin');
var ExtractTextPlugin = require('extract-text-webpack-plugin');
var OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin');

var env = config[process.env.NODE_ENV].env;
var webpackConfig = merge(baseWebpackConfig, {
  module: {
    rules: utils.styleLoaders({
      sourceMap: config[process.env.NODE_ENV].productionSourceMap,
      extract: true
    })
  },
  devtool: config[process.env.NODE_ENV].productionSourceMap ? '#source-map' : false,
  output: {
    path: config[process.env.NODE_ENV].assetsRoot,
    publicPath: '../',
    filename: '[name]/[name].js',
    chunkFilename: utils.assetsPath('js/[id].[chunkhash].js')
  },
  plugins: [
    // http://vuejs.github.io/vue-loader/en/workflow/production.html
    new webpack.DefinePlugin({
      'process.env': env
    }),

    // extract css into its own file
    new ExtractTextPlugin({
      filename: utils.assetsPath('../[name]/[name].css')
    }),
    // Compress extracted CSS. We are using this plugin so that possible
    // duplicated CSS from different components can be deduped.
    new OptimizeCSSPlugin(),
//    new webpack.optimize.CommonsChunkPlugin({
//      name: 'coui-public',
//      minChunks({resource}, count) {
//        if (count < 2) return false;
//        var chunks = Object.keys(baseWebpackConfig.entry);
//        return count >= chunks.length - 1;
//      }
//    }),
    // Dll library
    ...Object.keys(config.build.library).map(name => {
      return new webpack.DllReferencePlugin({
        context: '.',
        manifest: require(`${config.common.assetsPublicSourcePath}/dll/${name}.dll.manifest.json`)
      })
    })
  ]
});

for (var key in webpackConfig.entry) {
  if (key === config.common.publicDirectory) continue;
  webpackConfig.plugins.push(
    new HtmlWebpackPlugin({
      filename: key + '/index.html',
      template: 'index.html',
      chunks: [key],
      inject: true,
      minify: {
        removeComments: true,
        collapseWhitespace: true
        // removeAttributeQuotes: true
        // more options:
        // https://github.com/kangax/html-minifier#options-quick-reference
      },
      chunksSortMode: 'dependency'
    })
  );
}

if (config[process.env.NODE_ENV].cleanOutputDir) {
  var CleanWebpackPlugin = require('clean-webpack-plugin');
  // clean build or dist dir
  webpackConfig.plugins.push(new CleanWebpackPlugin([process.env.NODE_ENV], {
    root: path.resolve(__dirname, '../')
  }));
}

if (config[process.env.NODE_ENV].productionGzip) {
  var CompressionWebpackPlugin = require('compression-webpack-plugin');
  webpackConfig.plugins.push(
    new CompressionWebpackPlugin({
      asset: '[path].gz[query]',
      algorithm: 'gzip',
      test: new RegExp(
        '\\.(' +
        config[process.env.NODE_ENV].productionGzipExtensions.join('|') +
        ')$'
      ),
      threshold: 10240,
      minRatio: 0.8
    })
  );
}

if (process.env.NODE_ENV === 'dist') {
  webpackConfig.plugins.push(new webpack.optimize.UglifyJsPlugin({
    compress: {
      warnings: false
    },
    sourceMap: false
  }));
}

if (config[process.env.NODE_ENV].bundleAnalyzerReport) {
  var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;
  webpackConfig.plugins.push(new BundleAnalyzerPlugin());
}

module.exports = webpackConfig;
