let gulp = require('gulp')
  , fs = require('fs')
  , rimraf = require('gulp-rimraf')
  , zip = require('gulp-zip')
  , request = require('request')
  , config = require('./config')
  , server = require('./src/coui-public/config')

  , args = process.argv.splice(2)
  , index = args.findIndex(a => a === '-m') + 1
  , modules = index ? args[index].split(',') : []
  , dest = './zip'
  , pathname = 'api/v4'
  , JSESSIONID
  , MAPPING = {}
  ;

/*====================================
 =           clean dist Task         =
 ====================================*/

gulp.task('clean_dist', () => {
  return gulp.src(dest, {read: false}
  ).pipe(rimraf());
});

/*====================================
 =             Zip  Task             =
 ====================================*/

gulp.task('zip', done => {
  modules.forEach(i =>
    gulp.src([`${config.dist.assetsRoot}/${i}/**`], { base: config.dist.assetsRoot })
      .pipe(zip(`${i}.zip`))
      .pipe(gulp.dest(dest))
    );
  done();
});

/*====================================
 =            Login  Task            =
 ====================================*/

gulp.task('login', ['zip'], done => {
  request({
    url: `${server.mxm}/${pathname}/login`,
    method: 'POST',
    json: true,
    headers: {
      'content-type': 'application/json',
    },
    rejectUnauthorized: false,
    body: {
      deviceToken: '867247028634862',
      username: server.username,
      password: server.password
    }
  }, (error, response, body) => {
    if (error) {
      return console.error('failed:', error);
    } else if (response.statusCode === 200) {
      JSESSIONID = response.headers['set-cookie'][0].split(';')[0].split('=')[1];
      done();
    }
  });
});

/*====================================
 =            Zip  Upload            =
 ====================================*/

gulp.task('mapping', ['login'], done => {
  let j = request.jar();
  let cookie = request.cookie(`JSESSIONID=${JSESSIONID}`);
  j.setCookie(cookie, `${server.mxm}/light_function/list`);
  request({
    url: `${server.mxm}/light_function/list`,
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded'
    },
    jar: j,
    method: 'POST',
    rejectUnauthorized: false,
    form: {
      page: 1,
      limit: 1000,
      appName: ''
    }
  }, (error, response, body) => {
    if (error) {
      return console.error('failed:', error);
    } else if (response.statusCode === 200) {
      let list = JSON.parse(body).page.list;
      for (let i = 0, l = list.length; i < l; i++) {
        MAPPING[list[i].appCode.trim()] = {
          functionId: list[i].id,
          appName: list[i].appName
        }
      }
      done();
    }
  });
});

/*====================================
 =            Zip  Upload            =
 ====================================*/

gulp.task('upload', ['mapping'], function (done) {
  let j = request.jar();
  let cookie = request.cookie(`JSESSIONID=${JSESSIONID}`);
  let count = 0;
  j.setCookie(cookie, `${server.mxm}/${pathname}/attachment/upload`);
  for (let i = 0, l = modules.length; i < l; i++) {
    let module = modules[i];
    let map = MAPPING[module];
    console.log('准备上传模块' + module);
    if (!map) continue;
    let formData = {
      businessModel: 'light_function',
      file: {
        value: fs.createReadStream(`${dest}/${module}.zip`),
        options: {
          filename: `${module}.zip`,
          contentType: 'application/x-zip-compressed'
        }
      }
    };
    request.post({
      url: `${server.mxm}/${pathname}/attachment/upload`,
      rejectUnauthorized: false,
      formData: formData,
      jar: j
    }, (error, response, body) => {
      if (error) {
        return console.error('failed:', error);
      } else if (response.statusCode === 200) {
        body = JSON.parse(body);
        console.log(`${body.fileName} has upload`);
        save(body, map);
      }
      count++;
      if (count === l) {
        done();
      }
    });
  };

  /**
  * 关联文件
  */
  function save(file, map) {
    let cookie = request.cookie(`JSESSIONID=${JSESSIONID}`);
    j.setCookie(cookie, `${server.mxm}/function_version/save`);
    request.post({
      url: `${server.mxm}/function_version/save`,
      rejectUnauthorized: false,
      jar: j,
      json: true,
      headers: {
        'content-type': 'application/json',
      },
      body: {
      	'functionId': map.functionId,
      	'appName': map.appName,
      	'type': '1',
      	'url': '',
      	'file': [{
      		'uploadStatus': false,
      		'fileName': file.fileName,
      		'fileSize': file.fileSize,
      		'fileId': file.fileId,
      		'uid': new Date().getTime(),
      		'status': 'success'
      	}],
      	'address': file.fileId,
      	'remark': '',
      	'versionSize': file.fileSize
      }
    }, (error, response, body) => {
      if (error) {
        return console.error('failed:', error);
      } else if (response.statusCode === 200) {
        console.log(`successful! ${file.fileName} has publish`);
      }
    });
  }
});

/*====================================
 =            Zip  default           =
 ====================================*/

gulp.task('default', () => {
  console.log(`编译模块${modules}`);
});
