/**
 * @author umin <326663876@qq.com>, caixue, yangjianfeng, tangjian
 * @since
 * @copyright Copyright (c) 2018, YZKJ
 * @description 共用 js api
 * 重写 corJS，根据项目需要，修改或新增实例函数
 */
export default {

  /**
   * 示例，新增/修改，可参考 cor-native 用法一致
   * @param {Object} xxx 插件参数
   */
  ready: function() {
    // todo，重写 ready
    this.$super.ready(...arguments);
  },

  /**
   * 空函数
   */
  emptyMehod: function() {},

  /**
   * 深度copy
   * @returns {*|{}}
   */
  extend: function() {
    var options;
    var name;
    var src;
    var copy;
    var copyIsArray;
    var clone;
    var target = arguments[0] || {};
    var i = 1;
    var length = arguments.length;
    var deep = false;

    // Handle a deep copy situation
    if (typeof target === 'boolean') {
      deep = target;

      // Skip the boolean and the target
      target = arguments[i] || {};
      i++;
    }

    // Handle case when target is a string or something (possible in deep copy)
    if (typeof target !== 'object' && !this.isFunction(target)) {
      target = {};
    }

    // Extend jQuery itself if only one argument is passed
    if (i === length) {
      target = this;
      i--;
    }

    for (; i < length; i++) {

      // Only deal with non-null/undefined values
      if ((options = arguments[i]) != null) {

        // Extend the base object
        for (name in options) {
          src = target[name];
          copy = options[name];

          // Prevent never-ending loop
          if (target === copy) {
            continue;
          }

          // Recurse if we're merging plain objects or arrays
          if (deep && copy && (this.isPlainObject(copy) ||
            (copyIsArray = Array.isArray(copy)))) {

            if (copyIsArray) {
              copyIsArray = false;
              clone = src && Array.isArray(src) ? src : [];

            } else {
              clone = src && this.isPlainObject(src) ? src : {};
            }

            // Never move original objects, clone them
            target[name] = this.extend(deep, clone, copy);

            // Don't bring in undefined values
          } else if (copy !== undefined) {
            target[name] = copy;
          }
        }
      }
    }

    // Return the modified object
    return target;
  },
  class2type: function() {
    return {};
  },
  hasOwn: function() {
    return this.class2type().hasOwnProperty;
  },
  fnToString: function() {
    return this.hasOwn().toString;
  },
  ObjectFunctionString: function() {
    return this.fnToString().call(Object);
  },
  getProto: function() {
    return Object.getPrototypeOf;
  },
  isPlainObject: function(obj) {
    var proto, Ctor;

    // Detect obvious negatives
    // Use toString instead of jQuery.type to catch host objects
    if (!obj || toString.call(obj) !== '[object Object]') {
      return false;
    }

    proto = this.getProto(obj);

    // Objects with no prototype (e.g., `Object.create( null )`) are plain
    if (!proto) {
      return true;
    }

    // Objects with prototype are plain iff they were constructed by a global Object function
    Ctor = this.hasOwn.call(proto, 'constructor') && proto.constructor;
    return typeof Ctor === 'function' && this.fnToString.call(Ctor) === this.ObjectFunctionString;
  }

};
