/**
 * @author umin <326663876@qq.com>, caixue, yangjianfeng, tangjian
 * @since
 * @copyright Copyright (c) 2018, YZKJ
 * @description 原生插件调用层
 * 重写 corNative，根据项目需要，修改或新增实例函数
 */
const QUEUE = Cor.queue;
// 请求任务数
let requestTask = 0;
// 当前应用 appkey
let appKey;
// 获取应用appkey
const getAppKey = () => corMPortal.getAppKey();

const native = {

  /**
   * 示例：新增实例函数（框架没有的方法，包装新的方法）
   * 请求封装，处理请求参数及响应体解析
   * @param {String} params.url [必填] 请求地址
   * @param {String} params.method [选填] 请求方式 默认 GET
   * @param {String} params.headers [选填] 请求头
   * @param {String} params.data [选填] 请求参数
   * @param {String} params.timeout [选填] 请求超时
   * @param {Function} success [必填] 成功回调
   * @param {Function} fail [选填] 失败回调
   */
  request: function(setting, success, fail = corJS.emptyMehod) {
    // 本地静态数据模拟
    if (config.static) {
      let os = success;
      let key = setting.url;
      setting.url = 'assets/data.json';
      setting.method = 'get';
      success = data => os(data[key]);
      delete setting.data;
    } else if (!/^http/i.test(setting.url)) {
      // 调用接口，无域名则自动补全域名
      setting.url = config.mpm + setting.url;
    }
    // 添加门户默认请求头
    if (config.jwt) {
      if (!appKey) appKey = getAppKey();
      setting.headers = setting.headers || {};
      Object.assign(setting.headers, {
        'X-Authorization': config.jwt,
        'X-xSimple-appKey': appKey
      });
    };
    Cor.$indicator.open();
    requestTask++;
    // 发起异步请求
    this.requestAjax(setting, function(data) {
      let response = corJS.parse(data);
      if (response.status === true) {
        success(response);
      } else {
        fail(response);
        Cor.$toast(response.errorMessage || '获取数据失败！');
      }
      if (requestTask-- & requestTask === 0) Cor.$indicator.close();
    });
  },

  /**
   * 示例：修改实例函数（框架原有请求方法，修改请求逻辑）
   * 修改异步请求方法
   * @param {String} params.url [必填] 请求地址
   * @param {String} params.method [选填] 请求方式 默认 GET
   * @param {String} params.headers [选填] 请求头
   * @param {String} params.data [选填] 请求参数
   * @param {String} params.timeout [选填] 请求超时
   * @param {Function} callback 回调函数
   */
  requestAjax: function(param, callback) {
    if ((config.queue && param.queue !== false) || param.queue === true) {
      // 串行异步请求
      QUEUE.push({
        run: (resolve) => {
          this.$super.requestAjax(param, function(data) {
            resolve();
            callback(data);
          });
        }
      });
    } else {
      // 并行异步请求
      this.$super.requestAjax(...arguments);
    }
  },
  /**
   * 示例：新增实例函数（框架没有的方法，包装新的方法）
   * 附件上传
   * @param {String} params.url [选填] 请求地址 默认为门户上传接口地址
   * @param {String} params.headers [选填] 请求头
   * @param {String|Array} params.filePath [选填] 附件地址
   * @param {String} params.contentName [选填] 服务器接收附件的标识，默认:'file'
   * @param {Function} callback 回调函数
   */
  upload: function(setting, callback) {
    var datas = [];
    Cor.$indicator.open();
    if (Cor.isPC) {
      setTimeout(function() {
        Cor.$indicator.close();
      }, 500);
    }
    // 获取文件路径设置默认附件信息
    var files = corJS.isArray(setting.filePath) ? setting.filePath : corJS.isString(setting.filePath) ? [setting.filePath] : '';
    // 构建参数
    var createParam = function(filePath) {
      if (!filePath) return {};
      let file = {
        fPath: filePath,
        fType: filePath.split('.').pop(),
        fName: new Date().getTime() + '.' + filePath.split('.').pop()
      };
      let _setting = {
        url: !/^http/i.test(setting.url) ? config.mpm + '/upload-file' : setting.url,
        headers: setting.headers || {},
        data: { file: file },
        contentName: setting.contentName || 'files'
      };
      // 添加门户默认请求头
      if (config.jwt) {
        if (!appKey) appKey = getAppKey();
        Object.assign(_setting.headers, {
          'X-Authorization': config.jwt
        });
      };
      return _setting;
    };
    // 上传成功回调函数
    window.uploadSuccess = function(error, result) {
      if (result.data.reqCode === 2) {
        console.log('上传中...');
        Cor.$indicator.open();
      } else if (result && result.data.reqCode === 0) {
        // 上传成功
        console.log(result, error);
        Cor.$indicator.close();
        datas.push(corJS.parse(result.data.file));
        files.length ? corMPortal.upload(createParam(files.shift()), window.uploadSuccess) : callback(0, datas);
      } else if (error || result.data.reqCode === 1) {
        console.log('上传文件失败', result, error);
        Cor.$indicator.close();
        Cor.$toast(result.errorMsg || '上传文件失败');
        callback(1, corJS.parse(result));
      }
    };
    var param = createParam(files.shift());
    corMPortal.upload(param, window.uploadSuccess);
  }
};

export default native;
