/**
 * @author umin <326663876@qq.com>, caixue, yangjianfeng, tangjian
 * @since
 * @copyright Copyright (c) 2018, YZKJ
 * @description 同步状态
 */
/**
 * 动态添加 CSS 样式
 * @param selector {string} 选择器
 * @param rules    {string} CSS样式规则
 * @param index    {number} 插入规则的位置, 靠后的规则会覆盖靠前的，默认在后面插入
 */
let addCssRule = (function() {
  // 创建一个 style， 返回其 stylesheet 对象
  function createStyleSheet() {
    let style = document.createElement('style');
    style.type = 'text/css';
    document.head.appendChild(style);
    return style.sheet;
  }

  // 创建 stylesheet 对象
  let sheet = createStyleSheet();

  // 返回接口函数
  return function(selector, rules, index = 0) {
    sheet.insertRule(`${selector} {${rules}}`, index);
  };
})();

export {
  addCssRule
};
